#!/bin/bash
# © Copyright [2020] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

# Log files
TMP_LOG_FILE="/tmp/cis_media_upgrade.log"
DATE_NOW=$(date +%s)
LOG_FILE="/var/opt/novell/log/cis/cis_media_upgrade_$DATE_NOW.log"

SP3_0="sp3"

usage() {
  cat <<EOF
CIS Media upgrade utility
Usage: cis_media_media.sh <OPTIONS> CISOPTIONS
GENERAL OPTIONS (OPTIONS):
  -v, --version         Displays the program version information.
  -h, --help            Displays the help information.
CIS Options (CISOPTIONS):
  -p, --media_path     Specify the path where CIS configuration files and log files are stored. For example, specify /mnt or /media
EOF
}

version() {
  # Get Server version.
  oes_version=$(grep "VERSION" /etc/novell-release | cut -d'=' -f2- | xargs)
  echo -e "CIS media upgrade utility version $oes_version"
}

printandExit() {
  if [[ $1 -ne 0 ]]; then
    echo -e "CIS media upgrade failed."
    mv "$TMP_LOG_FILE" "$LOG_FILE"
    exit 1
  else
    echo -e "CIS media upgrade done successfully."
    mv "$TMP_LOG_FILE" "$LOG_FILE"
    exit 0
  fi
}

## Sets prefix path in configuration status file
updatePrefixPathInConfigStatus() {
  prefix_path=$1
  configStatus="/etc/opt/novell/cis/configurationStatus.json"
  INLINE_CODE="import json,sys;obj=json.load(open('$configStatus'));obj['prefixPath']=\"$prefix_path\"; print json.dumps(obj)"
  INLINE_CODE="import json,sys;obj=json.load(open('$configStatus'));obj['patch']=\"$SP3_0\"; print json.dumps(obj)"
  python -c "$INLINE_CODE" > "$configStatus".tmp
  mv "$configStatus".tmp "$configStatus"
}

## CIS standalone server
upgradeCisStandalone() {
  prefix_path=$1

  ## Stopping all CIS core services, If running
  declare -a cis_service_names=(
      "gateway" "mgmt"
      "collector" "policy"
      "repcollector" "aggregator"
      "metadata" "auth"
      "data" "repaggregator" 
      "configuration"
  )
  for name in "${cis_service_names[@]}"
  do
    echo -e "Stopping oes-cis-$name.service."
    systemctl stop oes-cis-"$name".service >> "$TMP_LOG_FILE" 2>&1
  done

  systemctl is-active --quiet oes-cis-zk.service >> "$TMP_LOG_FILE" 2>&1
  if [[ $? -ne 0 ]]; then
    echo -e "Starting CIS zookeeper service."
    systemctl start oes-cis-zk.service >> "$TMP_LOG_FILE" 2>&1
    systemctl is-active --quiet oes-cis-zk.service >> "$TMP_LOG_FILE" 2>&1
    if [[ $? -ne 0 ]]; then
      echo -e "Failed to start CIS zookeeper service."
      printandExit 1
    fi
  fi

  ZK_URI=$(grep "ZK_URI" /etc/opt/novell/cis/config | cut -d'=' -f2- | xargs)
  /opt/novell/cis/bin/cismigrate --update -zk_uri "$ZK_URI" -prefix_path "$prefix_path"  >> "$TMP_LOG_FILE" 2>&1
  if [[ $? -ne 0 ]]; then
    echo -e "Failed to update media path in Zookeeper."
    printandExit 1
  fi

  ## Stopping all CIS external dependencies, If running
  declare -a cis_ext_service_names=(
      "elastic" "kafka"
      "zk" "fluentbit"
  )
  for name in "${cis_ext_service_names[@]}"
  do
    echo -e "Stopping oes-cis-$name.service."
    systemctl stop oes-cis-"$name".service >> "$TMP_LOG_FILE" 2>&1
  done

  ## Fixing CIS prefix path
  echo -e "Moving the data and configuration file(s) to the media path."
  /opt/novell/cis/bin/cp2prefixpath.sh "$prefix_path" >> "$TMP_LOG_FILE" 2>&1
  if [[ $? -ne 0 ]]; then
    echo -e "Failed to move data and configuration file(s) to media path."
    printandExit 1
  fi

  updatePrefixPathInConfigStatus "$prefix_path"

  systemctl start oes-cis-zk.service >> "$TMP_LOG_FILE" 2>&1
  systemctl start oes-cis-elastic.service >> "$TMP_LOG_FILE" 2>&1
  systemctl start oes-cis-fluentbit.service >> "$TMP_LOG_FILE" 2>&1
  sleep 5
  systemctl start oes-cis-kafka.service >> "$TMP_LOG_FILE" 2>&1
  sleep 5

  ## Starting all CIS core services
  for name in "${cis_service_names[@]}"
  do
    echo -e "Starting oes-cis-$name.service."
    systemctl start oes-cis-"$name".service >> "$TMP_LOG_FILE" 2>&1
  done
  printandExit 0
}

## CIS core server
upgradeCisCore() {
  prefix_path=$1

  ## Stopping all CIS core services, If running
  declare -a cis_service_names=(
      "gateway" "mgmt"
      "collector" "policy"
      "repcollector" "aggregator"
      "metadata" "auth"
      "data" "repaggregator"
      "configuration"
  )
  for name in "${cis_service_names[@]}"
  do
    echo -e "Stopping oes-cis-$name.service."
    systemctl stop oes-cis-"$name".service >> "$TMP_LOG_FILE" 2>&1
  done

  ## Fixing CIS prefix path
  echo -e "Moving the data and configuration file(s) to the media path."
  /opt/novell/cis/bin/cp2prefixpath.sh "$prefix_path" >> "$TMP_LOG_FILE" 2>&1
  if [[ $? -ne 0 ]]; then
    echo -e "Failed to move data and configuration file(s) to media path."
    printandExit 1
  fi

  ZK_URI=$(grep "ZK_URI" /etc/opt/novell/cis/config | cut -d'=' -f2- | xargs)
  /opt/novell/cis/bin/cismigrate --update -zk_uri "$ZK_URI" -prefix_path "$prefix_path"  >> "$TMP_LOG_FILE" 2>&1
  if [[ $? -ne 0 ]]; then
    echo -e "Failed to update media path in Zookeeper."
    printandExit 1
  fi

  updatePrefixPathInConfigStatus "$prefix_path"

  ## Starting all CIS core services
  for name in "${cis_service_names[@]}"
  do
    echo -e "Starting oes-cis-$name.service."
    systemctl start oes-cis-"$name".service >> "$TMP_LOG_FILE" 2>&1
  done
  printandExit 0
}

## Update only patch version.
updatePatchInfoInConfigStatus(){
  configStatus="/etc/opt/novell/cis/configurationStatus.json"
  INLINE_CODE="import json,sys;obj=json.load(open('$configStatus'));obj['patch']=\"$SP3_0\"; print json.dumps(obj)"
  python -c "$INLINE_CODE" > "$configStatus".tmp
  mv "$configStatus".tmp "$configStatus"
}

writeDockerNwToConf(){
  conf="/etc/opt/novell/cis/config"
  subnet=$(docker network inspect -f "{{(index (.IPAM.Config) 0).Subnet}}" cisnet)
  if [ $? -ne 0 ]; then
    echo -e "Unable to get cisnet subnet." | tee -a "$TMP_LOG_FILE"    
    exit 1;
  fi

  gateway=$(docker network inspect -f "{{(index (.IPAM.Config) 0).Gateway}}" cisnet)
  if [ $? -ne 0 ]; then
    echo -e "Unable to get cisnet gateway." | tee -a "$TMP_LOG_FILE"    
    exit 1;
  fi

  sub=${subnet/\/*/}  

  IFS="."
  set -- $sub
  range=$((0x$(printf "%02x%02x%02x%02x\n" $1 $2 $3 $4)))
  set -- $gateway
  g=$4

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_DATA/ c\IP_DATA="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_METADATA/ c\IP_METADATA="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_POLICY/ c\IP_POLICY="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_AUTH/ c\IP_AUTH="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_MGMT/ c\IP_MGMT="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_COLLECTOR/ c\IP_COLLECTOR="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_AGGREGATOR/ c\IP_AGGREGATOR="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_REPCOLLECTOR/ c\IP_REPCOLLECTOR="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_REPAGGREGATOR/ c\IP_REPAGGREGATOR="'"$ip"'"' $conf

  g=$g+1
  v=$((range|g))
  ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
  sed -i '/IP_GATEWAY/ c\IP_GATEWAY="'"$ip"'"' $conf
 
  echo -e "Wrote new docker IP addresses." | tee -a "$TMP_LOG_FILE"

}

reconfigureDockerNetwork(){
  conf="/etc/opt/novell/cis/config"
  subnet=$(docker network inspect -f "{{(index (.IPAM.Config) 0).Subnet}}" cisnet)
  if [ $? -ne 0 ]; then
      echo -e "Unable to get cisnet subnet." | tee -a "$TMP_LOG_FILE"
      exit 1
  fi

  sub=${subnet/\/*/}

  if [ sub != "172.16.0.0" ]; then
    #stop services, remove docker network and recreate docker network
    stopServices
    if [ $? -ne 0 ]; then
      echo -e "Unable to stop cis services." | tee -a "$TMP_LOG_FILE"   
      exit 1         
    fi

    docker network rm cisnet 2>/dev/null
    if [ $? -ne 0 ]; then
      echo -e "Unable to remove docker network." | tee -a "$TMP_LOG_FILE"   
      exit 1         
    fi

    /bin/bash /opt/novell/cis/bin/activate_cis.sh

    writeDockerNwToConf
    if [ $? -ne 0 ]; then
      echo -e "Unable to write docker network to configuration file." | tee -a "$TMP_LOG_FILE"   
      exit 1         
    fi
  fi
}

add_dashboard(){
    config_Type=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'configType'\042/){print $(i+1)}}}' | tr -d '"' | xargs`

    if { [ "$config_Type" == "standalone" ] || [ "$config_Type" == "core" ]; };then
        conf='/etc/opt/novell/cis/config'
        repagg='REP_AGGREGATOR_NAME="cis-repaggregator"'

        dashboard='# Dashboard service Configuration\nDASHBOARD_NAME="oes-dashboard"\nDASHBOARD_THREADS="10"'

        #edit configuration file
        sed -i 's/'"$repagg"'/&\n'"$dashboard"'/' $conf

        #add to docker network
        gatewaystring=$(grep IP_GATEWAY "$conf")    
        dashboard=$(grep IP_GATEWAY "$conf"| cut -d'=' -f2 | xargs | cut -d'.' -f1-3 | xargs)
        dashboardip='IP_DASHBOARD="'"$dashboard"'.12"'
        sed -i 's/'"$gatewaystring"'/&\n'"$dashboardip"'/' $conf

        #load docker image
        docker load -i /usr/share/suse-docker-images/sles-oes-dashboard-*
        IMAGE_VERSION=`rpm -q --queryformat "%{VERSION}-%{RELEASE}" sles-oes-dashboard-docker-image`
        CUR_VERSION=`docker images microfocus/oes-dashboard | grep "microfocus/oes-dashboard" | awk -F ' ' '{print $2}'`
        docker tag microfocus/oes-dashboard:$CUR_VERSION microfocus/oes-dashboard:$IMAGE_VERSION

        systemctl enable oes-dashboard.service
        systemctl start oes-dashboard.service

        echo -e "Updated docker network for dashboard. \n" | tee -a "$TMP_LOG_FILE"
    fi
}

# Script entry-point
if [ "$1" == "" ]; then
  usage
  exit 1
fi

configuration_media_path=""
while [ "$1" != "" ]; do
  case $1 in
  --media_path)
    shift
    configuration_media_path=$1
    ;;
  -p)
    shift
    configuration_media_path=$1
    ;;
  --help)
    usage
    exit 0
    ;;
  -h)
    usage
    exit 0
    ;;
  --version)
    version
    exit 0
    ;;
  -v)
    version
    exit 0
    ;;
  *)
    usage
    exit 1
    ;;
  esac
  shift
done

## File not found. Check if it's linked and broken
if { [ -L /etc/opt/novell/cis ] && [ ! -e /etc/opt/novell/cis ]; }; then
  echo -e "NSS resource not mounted on the current node."
  exit 1
fi

## Check for configuration status file.
if { [ ! -f /etc/opt/novell/cis/configurationStatus.json ]; }; then  
    ## This shouldn't happen
    echo -e "File /etc/opt/novell/cis/configurationStatus.json not found."
    exit 1  
fi

SP2_U4_PATCH_VERSION_NAME="sp2update4"

## Get configuration status and type.
is_Configured=$(awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'cisConfigSuccess'\042/){print $(i+1)}}}' /etc/opt/novell/cis/configurationStatus.json | tr -d '"' | xargs)
config_Type=$(awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'configType'\042/){print $(i+1)}}}' /etc/opt/novell/cis/configurationStatus.json | tr -d '"' | xargs)
patch_version=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'patch'\042/){print $(i+1)}}}' | tr -d '"' | xargs`

## Check if already configured.
if { [ "$is_Configured" != "true" ]; }; then
  echo -e "CIS services not configured."
  exit 0
fi

# exit if already upgraded to sp3
if { [ $patch_version == $SP3_0 ]; }; then 
  echo -e "Media has been upgraded to SP3. Exiting."
  exit 0
fi

echo -e "Beginning CIS media upgrade." >>"$TMP_LOG_FILE" 2>&1
echo -e "CIS services configured in $config_Type mode" >>"$TMP_LOG_FILE" 2>&1

# copy from cis_local to cis
mv /etc/opt/novell/cis/configurationStatus.json /etc/opt/novell/cis/configurationStatus.json.backup 
mv /etc/opt/novell/cis/status.json /etc/opt/novell/cis/status.json.backup 

cp -rp /etc/opt/novell/cis_local /etc/opt/novell/cis/

mv /etc/opt/novell/cis/configurationStatus.json.backup /etc/opt/novell/cis/configurationStatus.json 
mv /etc/opt/novell/cis/status.json.backup /etc/opt/novell/cis/status.json

if { [ $patch_version == $SP2_U4_PATCH_VERSION_NAME ]; }; then  
  if { [ "$config_Type" == "standalone" ] || [ "$config_Type" == "core" ]; }; then
    echo -e "Patched to SP2 Update 4. Only adding dashboard service." 
    add_dashboard
    updatePatchInfoInConfigStatus
    printandExit 0
  fi
fi

if { [ "$config_Type" == "core" ] || [ "$config_Type" == "cluster" ]; };then
  reconfigureDockerNetwork
fi

if { [ "$config_Type" == "standalone" ]; }; then
  echo -e "Configuration Media path: $configuration_media_path"
  upgradeCisStandalone "$configuration_media_path"
fi

if { [ "$config_Type" == "core" ]; }; then
  echo -e "Configuration Media path: $configuration_media_path"
  upgradeCisCore "$configuration_media_path"
fi

if { [ "$config_Type" == "cluster" ]; }; then
  echo -e "Configuration Media path: $configuration_media_path"
  upgradeCisCore "$configuration_media_path"
fi

printandExit 0

